/* X Language - the eXtensible Language
 * Copyright (C) 2001 Patrick Deschenes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* These files are distributed at http://www.freesoftware.fsf.org/xlang/
 */

#include <xltype.h>
#include <xlmain.h>

XLType*
xl_type_new (XLTypeType p_type, XLTypeArrayType p_array_type, xuint p_array_size, XString* p_classname)
{
  XLType *self;

  self = x_new (XLType);
  x_object_init_object (X_OBJECT (self), xl_type_destroy);

  self->type = p_type;
  self->array_type = p_array_type;
  self->array_size = p_array_size;
  self->classname = x_addref (XString, p_classname);
  self->class = NULL;
  self->fnct = NULL;
    
  return self;
}

xvoid
xl_type_destroy (XObject* self)
{
  x_unref (XL_TYPE (self)->class);
  x_unref (XL_TYPE (self)->fnct);
  x_unref (XL_TYPE (self)->classname);

  x_destroy (self);
}

xvoid
xl_type_resolve (XLType* self)
{
  XLClass* l_class;
  XLFnct*  l_fnct;

  if (self->type == XL_TYPE_TYPE_UNKNOWN)
    {
      l_class = xl_main_get_class (g_xl_main_global, self->classname);
      self->type = XL_TYPE_TYPE_OBJECT;
      x_unref (l_class);
    }
}

xuint
xl_type_get_size (XLType* self)
{
  XLClass* l_class;

  switch (self->array_type)
    {
    case XL_TYPE_ARRAY_TYPE_NONE:
      switch (self->type)
	{
	case XL_TYPE_TYPE_STRING:
	  return sizeof (xuint) + sizeof (xptr);
	case XL_TYPE_TYPE_BOOL:
	  return sizeof (xbool);
	case XL_TYPE_TYPE_CHAR:
	  return sizeof (xchar);
	case XL_TYPE_TYPE_UCHAR:
	  return sizeof (xuchar);
	case XL_TYPE_TYPE_SHORT:
	  return sizeof (xshort);
	case XL_TYPE_TYPE_USHORT:
	  return sizeof (xushort);
	case XL_TYPE_TYPE_INT:
	  return sizeof (xint);
	case XL_TYPE_TYPE_UINT:
	  return sizeof (xuint);
	case XL_TYPE_TYPE_FLOAT:
	  return sizeof (xfloat);	
	case XL_TYPE_TYPE_DOUBLE:
	  return sizeof (xdouble);
	case XL_TYPE_TYPE_OBJECT:
	  return sizeof (XLDataObject);
	}
    case XL_TYPE_ARRAY_TYPE_FIXED:
      return sizeof (XLDataArray);
    case XL_TYPE_ARRAY_TYPE_DYNAMIC:
      return sizeof (XLDataArray);
    }

  printf ("unmanaged typed\n");
  return 0;
}

XLClass*
xl_type_get_class (XLType* self)
{
  if (self->type == XL_TYPE_TYPE_OBJECT)
    {
      if (!self->class)
	{
	  self->class = xl_main_get_class (g_xl_main_global, self->classname);
	  if (self->class)
	    return self->class;
	  else
	    {
	      printf ("error: undefined class '%s'\n", x_string_get_str (self->classname));
	      exit (0);
	    }
	}
      else
	return self->class;
    }
  else
    return NULL;
}

xvoid
xl_type_init_data (XLType* self, XLData* p_data)
{
}

xbool
xl_type_validate (XLType* self, XLType* p_type)
{
  return FALSE;
}


























